use std::io;

#[derive(Debug, PartialOrd, PartialEq)]
#[allow(non_camel_case_types)]
pub enum NodeManagementError {
    net_duplicate_packet    = 0x00,
    net_broken_packet       = 0x01,
    net_invalid_packet      = 0x02,
    net_addr_in_use         = 0x03,
    net_dest_unreachable    = 0x04,

    errc_not_registered     = 0x10,

    errs_not_delivered      = 0x20,
}

impl Into<u8> for NodeManagementError {
    fn into(self) -> u8 {
        self as u8
    }
}

impl TryFrom<u8> for NodeManagementError {
    type Error = io::Error;
    fn try_from(value: u8) -> io::Result<Self> {
        match value {
            0x00 => Ok(NodeManagementError::net_duplicate_packet),
            0x01 => Ok(NodeManagementError::net_broken_packet),
            0x02 => Ok(NodeManagementError::net_invalid_packet),
            0x03 => Ok(NodeManagementError::net_addr_in_use),
            0x04 => Ok(NodeManagementError::net_dest_unreachable),
            0x10 => Ok(NodeManagementError::errc_not_registered),
            0x20 => Ok(NodeManagementError::errs_not_delivered),
            _ => Err(io::Error::new(io::ErrorKind::InvalidInput, "non supported error"))
        }
    }
}