use crate::identifier::Identifier;
use crate::packet::Packet;
use crate::packet::packet_data::PacketData;
use crate::packet::raw_packet::RawPacket;

impl From<Packet> for RawPacket {
    fn from(packet: Packet) -> Self {
        let data = &packet.data.into_bytes();
        let data_length = data.len();
        RawPacket {
            version: 0x01,
            dest: packet.dest.into(),
            src: packet.src.into(),
            packet_id: packet.packet_id.into(),
            reply_to: packet.reply_to.into(),
            command: packet.command,
            data_length,
            data: data.clone(),
            checksum: {
                let mut v = vec![0x01];
                v.extend(packet.dest.bytes);
                v.extend(packet.src.bytes);
                v.extend(packet.packet_id.bytes);
                v.extend(packet.reply_to.bytes);
                v.push(packet.command);
                v.extend((data_length as u16).to_be_bytes());
                v.extend(data);
                crc32fast::hash(v.as_ref())
            }
        }
    }
}

impl Into<Packet> for RawPacket {
    fn into(self) -> Packet {
        Packet {
            dest: Identifier::from(self.dest),
            src: Identifier::from(self.src),
            packet_id: Identifier::from(self.packet_id),
            reply_to: Identifier::from(self.reply_to),
            command: self.command,
            data: PacketData::try_from(self.clone())
                .map_err(|e| {
                    println!("{}", e);
                    e
                })
                .unwrap_or(PacketData::Unknown(self.data)),
        }
    }
}